@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -187,8 +187,8 @@ void flb_kube_conf_destroy(struct flb_kube *ctx)
     }
 
 #ifdef FLB_HAVE_TLS
-    if (ctx->tls.context) {
-        flb_tls_context_destroy(ctx->tls.context);
+    if (ctx->tls) {
+        flb_tls_destroy(ctx->tls);
     }
 #endif
 
