@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -41,6 +41,10 @@ struct flb_elasticsearch {
     char *http_user;
     char *http_passwd;
 
+    /* Elastic Cloud Auth */
+    char *cloud_user;
+    char *cloud_passwd;
+
     /* AWS Auth */
 #ifdef FLB_HAVE_AWS
     int has_aws_auth;
@@ -49,9 +53,9 @@ struct flb_elasticsearch {
     struct flb_aws_provider *aws_provider;
     struct flb_aws_provider *base_aws_provider;
     /* tls instances can't be re-used; aws provider requires a separate one */
-    struct flb_tls aws_tls;
+    struct flb_tls *aws_tls;
     /* one for the standard chain provider, one for sts assume role */
-    struct flb_tls aws_sts_tls;
+    struct flb_tls *aws_sts_tls;
     char *aws_session_name;
 #endif
 
@@ -102,6 +106,8 @@ struct flb_elasticsearch {
     /* Elasticsearch HTTP API */
     char uri[256];
 
+    struct flb_record_accessor *ra_prefix_key;
+
     /* Upstream connection to the backend server */
     struct flb_upstream *u;
 
