@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -103,14 +103,15 @@ struct flb_s3 {
     struct flb_aws_provider *provider;
     struct flb_aws_provider *base_provider;
     /* tls instances can't be re-used; aws provider requires a separate one */
-    struct flb_tls provider_tls;
+    struct flb_tls *provider_tls;
     /* one for the standard chain provider, one for sts assume role */
-    struct flb_tls sts_provider_tls;
-    struct flb_tls client_tls;
+    struct flb_tls *sts_provider_tls;
+    struct flb_tls *client_tls;
 
     struct flb_aws_client *s3_client;
     int json_date_format;
     flb_sds_t json_date_key;
+    flb_sds_t date_key;
 
     flb_sds_t buffer_dir;
 
