@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,12 +23,12 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_unescape.h>
 #include <fluent-bit/flb_utils.h>
-
-#include <jsmn/jsmn.h>
+#include <fluent-bit/flb_jsmn.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "gce_metadata.h"
 #include "stackdriver.h"
 #include "stackdriver_conf.h"
 
@@ -183,6 +183,22 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
     ctx->ins = ins;
     ctx->config = config;
 
+    /* Lookup metadata server URL */
+    tmp = flb_output_get_property("metadata_server", ctx->ins);
+    if(tmp) {
+        ctx->metadata_server = flb_sds_create(tmp);
+    }
+    else {
+        tmp = getenv("METADATA_SERVER");
+        if(tmp) {
+            ctx->metadata_server = flb_sds_create(tmp);
+        }
+        else {
+            ctx->metadata_server = flb_sds_create(FLB_STD_METADATA_SERVER);
+        }
+    }
+    flb_plg_info(ctx->ins, "metadata_server set to %s", ctx->metadata_server);
+
     /* Lookup credentials file */
     tmp = flb_output_get_property("google_service_credentials", ins);
     if (tmp) {
@@ -280,7 +296,7 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
     else {
         ctx->autoformat_stackdriver_trace = FLB_FALSE;
     }
-    
+
     tmp = flb_output_get_property("trace_key", ins);
     if (tmp) {
         ctx->trace_key = flb_sds_create(tmp);
@@ -298,13 +314,13 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
     }
 
     if (flb_sds_cmp(ctx->resource, "k8s_container",
-                    flb_sds_len(ctx->resource)) == 0 || 
-        flb_sds_cmp(ctx->resource, "k8s_node", 
                     flb_sds_len(ctx->resource)) == 0 ||
-        flb_sds_cmp(ctx->resource, "k8s_pod", 
+        flb_sds_cmp(ctx->resource, "k8s_node",
+                    flb_sds_len(ctx->resource)) == 0 ||
+        flb_sds_cmp(ctx->resource, "k8s_pod",
                     flb_sds_len(ctx->resource)) == 0) {
-        
-        ctx->k8s_resource_type = FLB_TRUE;
+
+        ctx->is_k8s_resource_type = FLB_TRUE;
 
         tmp = flb_output_get_property("k8s_cluster_name", ins);
         if (tmp) {
@@ -324,6 +340,65 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
         }
     }
 
+    if (flb_sds_cmp(ctx->resource, "generic_node",
+                    flb_sds_len(ctx->resource)) == 0 ||
+        flb_sds_cmp(ctx->resource, "generic_task",
+                    flb_sds_len(ctx->resource)) == 0) {
+
+        ctx->is_generic_resource_type = FLB_TRUE;
+
+        tmp = flb_output_get_property("location", ins);
+        if (tmp) {
+            ctx->location = flb_sds_create(tmp);
+        } else {
+            flb_plg_error(ctx->ins, "missing generic resource's location");
+        }
+
+        tmp = flb_output_get_property("namespace", ins);
+        if (tmp) {
+            ctx->namespace_id = flb_sds_create(tmp);
+        } else {
+            flb_plg_error(ctx->ins, "missing generic resource's namespace");
+        }
+
+        if (flb_sds_cmp(ctx->resource, "generic_node",
+                    flb_sds_len(ctx->resource)) == 0) {
+            tmp = flb_output_get_property("node_id", ins);
+            if (tmp) {
+                ctx->node_id = flb_sds_create(tmp);
+            } else {
+                flb_plg_error(ctx->ins, "missing generic_node's node_id");
+                flb_stackdriver_conf_destroy(ctx);
+                return NULL;
+            }
+        }
+        else {
+            tmp = flb_output_get_property("job", ins);
+            if (tmp) {
+                ctx->job = flb_sds_create(tmp);
+            } else {
+                flb_plg_error(ctx->ins, "missing generic_task's job");
+            }
+
+            tmp = flb_output_get_property("task_id", ins);
+            if (tmp) {
+                ctx->task_id = flb_sds_create(tmp);
+            } else {
+                flb_plg_error(ctx->ins, "missing generic_task's task_id");
+            }
+
+            if (!ctx->job || !ctx->task_id) {
+                flb_stackdriver_conf_destroy(ctx);
+                return NULL;
+            }
+        }
+
+        if (!ctx->location || !ctx->namespace_id) {
+            flb_stackdriver_conf_destroy(ctx);
+            return NULL;
+        }
+    }
+
     tmp = flb_output_get_property("labels_key", ins);
     if (tmp) {
         ctx->labels_key = flb_sds_create(tmp);
@@ -337,7 +412,7 @@ struct flb_stackdriver *flb_stackdriver_conf_create(struct flb_output_instance *
         ctx->tag_prefix = flb_sds_create(tmp);
     }
     else {
-        if (ctx->k8s_resource_type == FLB_TRUE) {
+        if (ctx->is_k8s_resource_type == FLB_TRUE) {
             ctx->tag_prefix = flb_sds_create(ctx->resource);
         }
     }
@@ -351,7 +426,7 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
         return -1;
     }
 
-    if (ctx->k8s_resource_type){
+    if (ctx->is_k8s_resource_type){
         flb_sds_destroy(ctx->namespace_name);
         flb_sds_destroy(ctx->pod_name);
         flb_sds_destroy(ctx->container_name);
@@ -361,6 +436,19 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
         flb_sds_destroy(ctx->local_resource_id);
     }
 
+    if (ctx->is_generic_resource_type){
+        flb_sds_destroy(ctx->location);
+        flb_sds_destroy(ctx->namespace_id);
+        if(ctx->node_id){
+            flb_sds_destroy(ctx->node_id);
+        }
+        else {
+            flb_sds_destroy(ctx->job);
+            flb_sds_destroy(ctx->task_id);
+        }
+    }
+
+    flb_sds_destroy(ctx->metadata_server);
     flb_sds_destroy(ctx->credentials_file);
     flb_sds_destroy(ctx->type);
     flb_sds_destroy(ctx->project_id);
@@ -377,10 +465,6 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
     flb_sds_destroy(ctx->labels_key);
     flb_sds_destroy(ctx->tag_prefix);
 
-    if (ctx->o) {
-        flb_oauth2_destroy(ctx->o);
-    }
-
     if (ctx->metadata_server_auth) {
         flb_sds_destroy(ctx->zone);
         flb_sds_destroy(ctx->instance_id);
@@ -393,6 +477,11 @@ int flb_stackdriver_conf_destroy(struct flb_stackdriver *ctx)
     if (ctx->u) {
         flb_upstream_destroy(ctx->u);
     }
+
+    if (ctx->o) {
+        flb_oauth2_destroy(ctx->o);
+    }
+
     flb_free(ctx);
 
     return 0;
