@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -251,55 +251,5 @@ void flb_router_exit(struct flb_config *config)
     }
 }
 
-/*
- * Calculate the routes_mask for input chunk with a router_match on tag
- */
-uint64_t flb_router_get_routes_mask_by_tag(const char *tag, int tag_len,
-                                           struct flb_input_instance *in) {
-    uint64_t routes_mask = 0;
-    struct mk_list *o_head;
-    struct flb_output_instance *o_ins;
-    if (!in) {
-        return -1;
-    }
 
-    /* Find all matching routes for the given tag */
-    mk_list_foreach(o_head, &in->config->outputs) {
-        o_ins = mk_list_entry(o_head,
-                              struct flb_output_instance, _head);
 
-        if (flb_router_match(tag, tag_len, o_ins->match
-#ifdef FLB_HAVE_REGEX
-                             , o_ins->match_regex
-#else
-                             , NULL
-#endif
-                             )) {
-            /*
-             * mask_id for each output instance is a unique number starting from 1
-             * and multple by 2 each time. (e.g 1, 2 ,4 ,8, 16 ...)
-             * Let's take a look of the binary of the mask_id:
-             *   1:   00000001
-             *   2:   00000010
-             *   4:   00000100
-             *   8:   00001000
-             *   16:  00010000
-             * We can notice that each binary has only one 1's bit and this also
-             * represents the postion of the output instance. Getting the OR of
-             * mask_id (given that tag is matched) will tell us the output instances
-             * that the given input chunk will flush to.
-             *
-             * For example: We have two matching output instances with mask_id 1 and 4
-             * There are two 1's in the binary with index 0 and 2 (starting from right)
-             * and this means that the input chunk will flush to first and third output
-             * instances configured in the Fluent Bit configuraion.
-             *
-             *    0 |= 1 -> 00000 |= 00001 -> 00001
-             *    00001 |= 4 -> 00001 |= 00100 -> 00101
-             */
-            routes_mask |= o_ins->mask_id;
-        }
-    }
-
-    return routes_mask;
-}
